import re
import pkg_resources
import os

version = pkg_resources.require("jigoku")[0].version


class Jigoku:
        
    @staticmethod
    def extract_img_name(url) -> str:
        """Extract image name from url

        Parameters
        ----------
        url : str
            Image url

        Returns
        -------
        str

        """

        image_name = url.split("/")[-1]
        image_name = image_name.split("?")[0]
        
        if len(image_name) > 200:
            ext = image_name.split(".")[-1]
            image_name = image_name[:200 - len(ext) - 1] + "." + ext

        return image_name


    @staticmethod
    def proper_protocols(url: str) -> str:
        if url.startswith("//"):
            return url.replace("//", "https://", 1)
        else:
            return url

    @staticmethod
    def get_href_value(string) -> str:
        """Get href value from long soup

        Parameters
        ----------
        string : str
            String to get href value

        Returns
        -------
        str

        """
        sampah_1 = re.sub(r"#", "", string)
        sampah_2 = re.sub(r"javascript:;", "", sampah_1)
        ## replace all " " with "%20"
        sampah_3 = re.sub(r" ", "%20", sampah_2)
        urls = re.findall(r'href=[\'"]?([^\'" >]+)', sampah_3)
        data = ", ".join(urls)
        return Jigoku.proper_protocols(data)

    @staticmethod
    def proper_yandere_link(string) -> str:
        """Proper yandere link smh

        Parameters
        ----------
        string : str
            String to proper

        Returns
        -------
        str

        """
        string = re.sub(r".*https://", "https://", string)
        string = re.sub(r".*http://", "http://", string)
        return string

    @staticmethod
    def change_protocol(url) -> str:
        """Change http to https, lolibooru doing this shit

        Parameters
        ----------
        url : str
            String to change

        Returns
        -------
        str

        """
        return url.replace("http://", "https://", 1)

    @staticmethod
    def validate_links(url: str) -> bool:
        ## print(f"Mocking... {url.strip()}")
        if (
            not "/posts/" in url ## danbooru & furry based
            and not "&id=" in url ## gelbooru based
            and not "?id=" in url ## gelbooru based
            and not "/show" in url ## yandere based
            and not "/post/view/" in url ## paheal based
        ):   
            return False
        else:
            return True
        

    def __init__(
        self,
        Gelbooru: str = "https://gelbooru.com",
        Safebooru: str = "https://safebooru.org",
        Danbooru: str = "https://danbooru.donmai.us",
        Rule34: str = "https://rule34.xxx",
        Tbib: str = "https://tbib.org",
        Xbooru: str = "https://xbooru.com",
        Realbooru: str = "https://realbooru.com",
        Yandere: str = "https://yande.re",
        Lolibooru: str = "https://lolibooru.moe",
        Konachan: str = "https://konachan.com",
        KonachanNet: str = "https://konachan.net",
        Hypnohub: str = "https://hypnohub.net",
        E621: str = "https://e621.net",
        E926: str = "https://e926.net",
        Allthefallen: str = "https://booru.allthefallen.moe",
        Paheal: str = "https://rule34.paheal.net",
        BASE_headers: dict = {
            "User-Agent": f"jigoku/v{version} (https://pypi.org/project/jigoku);",
            "From": "hey@sinkaroid.org",
        },
        Current_dir: str = os.getcwd(),
        Version: str = version,
        Expected_format: tuple = (
            ".jpg",
            ".png",
            ".gif",
            ".webm",
            ".mp4",
            ".jpeg",
        ),
    ):

        self.gelbooru = Gelbooru
        self.safebooru = Safebooru
        self.danbooru = Danbooru
        self.rule34 = Rule34
        self.tbib = Tbib
        self.xbooru = Xbooru
        self.realbooru = Realbooru
        self.yandere = Yandere
        self.lolibooru = Lolibooru
        self.konachan = Konachan
        self.konachanNet = KonachanNet
        self.hypnohub = Hypnohub
        self.e621 = E621
        self.e926 = E926
        self.allthefallen = Allthefallen
        self.paheal = Paheal
        self.supported = [
            Gelbooru,
            Safebooru,
            Danbooru,
            Rule34,
            Tbib,
            Xbooru,
            Realbooru,
            Yandere,
            Lolibooru,
            Konachan,
            KonachanNet,
            Hypnohub,
            E621,
            E926,
            Allthefallen,
            Paheal,
        ]
        self.with_headers = BASE_headers
        self.current_dir = Current_dir
        self.version = Version
        self.expected_format = Expected_format



import logging
import os
logging.basicConfig(format="%(asctime)s - %(message)s", level=logging.INFO)

def log_data(case, note) -> str:
    """Logging data
    Parameters
    ----------
    case : str
        Case of the log
    note : str
        Note of the log
    """
    logging.info(f"{case} {note}")

def log_time(start, end) -> str:
    """Logging time taken

    Parameters
    ----------
    start : float
        Initial start
    end : float
        End date
    """
    logging.info("Task finished took {} min, {} sec".format(int((end - start) / 60), int((end - start) % 60)))
    
def get_hostname(url) -> str:
    """Get hostname from url
    Parameters
    ----------
    url : str
        URL
    Returns
    -------
    hostname : str
        Hostname
    """
    return f'https://{url.split("/")[2]}'
    


import os
import sys
import time
from inputimeout import inputimeout, TimeoutOccurred
from jigoku.utils.log import log_time
from jigoku.utils.scrape_posts import download_from_multiple_posts
from jigoku.utils.scrape_pages import download_from_multiple_pages


# from jigoku.utils.log import log_time
## which is takes all galleries

## from utils.log import log_time

try:
    file = inputimeout(
        prompt="[+] Please enter file you want to bulk download (e.g. file.txt): ", 
        timeout=30
    )
    
    download_by = inputimeout(
        prompt="[+] Please choose (1) Multiple posts or (2) Multiple pages: ",
        timeout=30,
    )
    
    select_type = inputimeout(
        prompt="[+] Select type image results (1) Original size or (2) Smaller size: ", 
        timeout=30
    )

except TimeoutOccurred:
    print("Timeout occurred, kindly read the docs: https://github.com/sinkaroid/jigoku#usage")
    os._exit(0)

if not file.endswith(".txt"):
    file = file + ".txt"


def main():
    if download_by == "1" or download_by == "posts" or download_by == "post":
        start = time.time()
        download_from_multiple_posts(file, select_type)
        end = time.time()
        log_time(start, end)

    elif download_by == "2" or download_by == "pages" or download_by == "page":
        start = time.time()
        download_from_multiple_pages(file, select_type)
        end = time.time()
        log_time(start, end)

    else:
        print("Invalid request")
        os._exit(0)


if __name__ == "__main__":
    try:
        main()
    except KeyboardInterrupt:
        print("Interrupted")
        try:
            sys.exit(0)
        except SystemExit:
            os._exit(0)



import os
import re
CLEANR = re.compile('<.*?>') 

def get_size(string) -> float:
    """Get size of a file
    Parameters
    ----------
    string : str
        Path to file
    Returns
    -------
    file : int
        Size of the file
    """
    file = round(os.path.getsize(string) / 1024 / 1024, 2)
    return file

def clean_html(raw_html) -> str:
    """Cleaning some shit
    Parameters
    ----------
    raw_html : str
        String to clean
    Returns
    -------
    cleantext : str
        Cleaned string

    """
    cleantext = re.sub(CLEANR, '', raw_html)
    cleantext = cleantext.strip()
    return cleantext
    


import requests
import os
import time
import re
import pkg_resources
from bs4 import BeautifulSoup
from jigoku.utils.log import log_data
from jigoku.utils.disk import get_size
from jigoku.utils.constant import Jigoku
version = pkg_resources.require("jigoku")[0].version

# from jigoku.utils.log import log_data, get_hostname

jgx = Jigoku()


def download_from_multiple_posts(file: str, select_type: str) -> None:
    """Bulk request download from multiple posts or galleries
    Parameters
    ----------
    file : str
        Path to file
    select_type : str
        Type of image to download

    """
    img_count = 0
    img_already_exist = 0

    with open(f"{file}", "r") as f:
        for line in f:
            if not line.strip():
                continue
            elif not line.startswith(tuple(jgx.supported)):
                raise Exception(f"Unsupported site {line}")

            elif line.startswith(jgx.e926):
                raise Exception(line.strip(), "Change this to e621 instead")

            elif not jgx.validate_links(line):
                raise Exception(f"Invalid links {line.strip()}, expected posts or galleries links, not pages")

            total_lines = sum(1 for line in open(f"{file}"))
            while True:
                try:
                    res = requests.get(line.replace("\n", ""), headers=jgx.with_headers)
                    if res.status_code == 200:
                        soup = BeautifulSoup(res.text, "html.parser")

                        if (
                            line.startswith(jgx.safebooru)
                            or line.startswith(jgx.tbib)
                            or line.startswith(jgx.xbooru)
                            or line.startswith(jgx.hypnohub)
                            or line.startswith(jgx.rule34)
                        ):
                            try:
                                if (line.startswith(jgx.hypnohub) or line.startswith(jgx.rule34)):
                                    first = str(soup.find("div", class_="link-list").find_all("li"))
                                    struktur_kontol = first.replace(" Original image", "Original image") ## anjing

                                    get_image_original = re.search(r'<a(.+?)>\nOriginal image', struktur_kontol).group(1)
                                    image_original = Jigoku.get_href_value(get_image_original)
                                    second = soup.find("div", class_="content").find("img")      
                                    image_small = Jigoku.proper_protocols(second["src"].split("?")[0])
                                else:
                                    first = str(soup.find("div", class_="sidebar").find_all("li"))
                                    get_image_original = re.search(r'<li>(.+?)>Original image</a>', first).group(1)
                                    image_original = Jigoku.get_href_value(get_image_original)
                                    second = soup.find("div", class_="content").find("img")      
                                    image_small = Jigoku.proper_protocols(second["src"].split("?")[0])
                            except:
                                first = soup.find("div", class_="content").find("source")
                                image_original = Jigoku.proper_protocols(first["src"].split("?")[0])
                                image_small = image_original

                        elif line.startswith(jgx.danbooru) or line.startswith(jgx.allthefallen):
                                first_if_fail = soup.find("section", id="post-options").find("li", id="post-option-download")
                                image_original = Jigoku.get_href_value(str(first_if_fail.find("a"))).split("?")[0]
                                image_small = image_original

                        elif line.startswith(jgx.gelbooru):
                            try:
                                first = str(soup.find("section", class_="aside").find_all("li"))
                                get_image_original = re.search(r'<li>(.+?)>Original image</a>', first).group(1)
                                image_original = Jigoku.get_href_value(get_image_original)
                                image_small = soup.find("picture").find("img")["src"]
                            except:
                                first = str(soup.find("section", class_="aside").find_all("li"))
                                get_image_original = re.search(r'<li>(.+?)>Original image</a>', first).group(1)
                                image_original = Jigoku.get_href_value(get_image_original)
                                image_small = image_original

                        elif line.startswith(jgx.realbooru):
                            first = soup.find("div", style="text-align: right; font-size: .7em;").find("a")
                            image_original = first["href"]
                            image_small = image_original

                        elif (
                            line.startswith(jgx.yandere)
                            or line.startswith(jgx.lolibooru)
                            or line.startswith(jgx.konachan)
                            or line.startswith(jgx.konachanNet)
                        ):
                            first = str(soup.find("div", class_="sidebar").find_all("li"))
                            get_image_original = re.search(r'<li><a class="original(.+?)>View larger version', first).group(1)
                            image_original = Jigoku.get_href_value(get_image_original)
                            get_image_small = soup.find("div", class_="content").find("img", alt=True)
                            image_small = get_image_small["src"]

                        elif line.startswith(jgx.e621):
                            try:
                                first = soup.find("div", id="image-download-link").find("a")
                                image_original = first["href"]
                                second = soup.find("img", id="image", class_="fit-window")
                                image_small = second["src"]
                            except:
                                first = soup.find("div", id="image-download-link").find("a")
                                image_original = first["href"]
                                image_small = image_original

                        elif line.startswith(jgx.paheal):
                            try:
                                first = soup.find("section", id="Imagemain").find("img")
                                image_original = first["src"]
                                image_small = image_original
                            except:
                                first = soup.find("section", id="Videomain").find("a")
                                image_original = first["href"]
                                image_small = image_original

                        if (
                            select_type == "1"
                            or select_type == "o"
                            or select_type == ""
                            or select_type == "original"
                            or select_type == "ori"
                        ):
                            final_img = image_original
                            final_name = Jigoku.extract_img_name(image_original)
                        elif (
                            select_type == "2"
                            or select_type == "s"
                            or select_type == "smaller"
                            or select_type == "small"
                        ):
                            final_img = image_small
                            final_name = Jigoku.extract_img_name(image_small)
                        else:
                            print("Invalid type")
                            exit()

                        ##print(final_name, final_img)
                        try:
                            if not os.path.exists(final_name):
                                img_count += 1
                                ##current_progress = f"{img_count} / {len(f.readlines()) + 1}"
                                image = requests.get(final_img, stream=True)
                                with open(final_name, "wb") as f:
                                    f.write(image.content)
                                
                                log_data(
                                    f'{img_count} / {total_lines} | {line.split("/")[2]}',
                                    f"{final_name} | Downloaded {get_size(final_name)} MB",
                                )
                                break

                            elif os.path.exists(final_name):
                                img_already_exist += 1
                                log_data(
                                    "Skipping... File already exists {}".format(
                                        final_name
                                    ),
                                    img_already_exist,
                                )
                                break

                        except:
                            print(f"Error: {final_name, final_img} failed to download")

                    else:
                        print(f"Error: {res.status_code}")
                        break

                except Exception as e:
                    if 'NoneType' in str(e):
                        print(f"{line} Skipping with no continue because {e} | Check failed.log for more info")
                        img_already_exist += 1
                        with open("failed.log", "a") as f:
                            f.write(line + "\n")
                        break
                    else:
                        print(f"{line} Retrying because {e} in 3 seconds.. | {res.status_code}")
                        time.sleep(3)
                        continue
                
        log_data(f"Downloaded {img_count} contents", f"Skipped {img_already_exist} contents")



__version__ = "2.2.2"
from .app import main
from .utils.log import log_data, log_time, get_hostname
from .utils.disk import get_size, clean_html
from .utils.scrape_posts import download_from_multiple_posts
from .utils.scrape_pages import download_from_multiple_pages